<?php

// exit if accessed directly
if( ! defined( 'ABSPATH' ) ) exit;

class ReyCoreElementor_Widget_Assets
{
	private static $_instance = null;
	private $reycore_elementor = null;
	private $scripts = [];
	private $styles = [];
	private $added_elements = [];

	const PREFIX = 'reycore-';
	const ASSET_PREFIX = 'reycore-widget-';
	const ASSET_STYLE_SUFFIX = '-styles';
	const ASSET_SCRIPT_SUFFIX = '-scripts';

	private $styles_to_load_natural_mode = [];

	private function __construct()
	{
		$this->reycore_elementor = reyCoreElementor();

		add_action('init', [$this, 'get_scripts_and_styles']);

		add_action( 'reycore/assets/register_scripts', [$this, 'register_widgets_assets']);

		// is it needed anymore?
		add_action( 'elementor/element/before_parse_css', [$this, 'force_newly_added_elements_into_header'], 0, 2 );

		add_action( 'elementor/element/parse_css', [$this, 'add_section_custom_css_to_post_file'], 10, 2 );

		add_action( 'reycore/elementor/global_section/ajax_load', [$this, 'load_ajax_assets'], 10, 2 );
	}

	/**
	 * These are widgets that are likely to be added above the fold.
	 * Because of this, they should be loaded in the <head> tag,
	 * but only when for some reason (such as a caching plugin but without minify/combine is installed),
	 * because otherwise Rey takes care of them instead and loads them properly.
	 * To be refactored and add them inside widgets.
	 *
	 * @return array
	 */
	public function high_priority_styles(){
		return [
			'cover-skew',
			'cover-panels',
			'cover-blurry',
			'cover-sideslide',
			'cover-double-slider',
			'cover-split',
			'cover-nest',
			'revolution-slider',
		];
	}

	/**
	 * Sets the high priority styles that should be loaded inside the <head> tag,
	 * but only when Caching plugins don't have minify/combine enabled.
	 *
	 * @return void
	 */
	function get_styles_to_load_natural($assets){

		if( ! $assets->settings['load_natural_css'] ){
			return;
		}

		if( ! class_exists('\Elementor\Plugin') ){
			return;
		}

		if ( ! $document = \Elementor\Plugin::instance()->documents->get( get_the_ID() ) ) {
			return;
		}

		$data = $document->get_elements_data();

		if ( empty( $data ) ) {
			return;
		}

		\Elementor\Plugin::instance()->db->iterate_data( $data, [$this, '__iterate_elements']);

	}

	public function __iterate_elements($element){

		if ( empty( $element['widgetType'] ) ){
			return;
		}

		$name = str_replace(self::PREFIX, '', $element['widgetType']);

		if( ! in_array($name, $this->high_priority_styles(), true) ){
			return;
		}

		$this->styles_to_load_natural_mode[] = self::ASSET_PREFIX . $name . self::ASSET_STYLE_SUFFIX;

	}


	/**
	 * Get all script.js and style.css files.
	 * TODO REFACTOR
	 * Each widget must have its own `register_assets` method.
	 *
	 * @since 1.0.0
	 */
	public function get_scripts_and_styles(){

		$disabled_elements = class_exists('ReyCore_WidgetsManager') ? ReyCore_WidgetsManager::get_disabled_elements() : [];

		foreach ( $this->reycore_elementor->widgets as $widget ) {

			// don't load disabled elements
			if( in_array(self::PREFIX . $widget, $disabled_elements, true) ){
				continue;
			}

			$assets_folder = trailingslashit('assets');

			$dir_path = reycore__clean( trailingslashit($this->reycore_elementor->widgets_dir . $widget) . $assets_folder );
			$uri = reycore__clean( trailingslashit(REY_CORE_URI . $this->reycore_elementor->widgets_folder . $widget) . $assets_folder );

			// script
			if( is_readable($dir_path . 'script.js') ){
				$this->scripts[$widget] = $uri . 'script.js';
			}

			if( is_readable($dir_path . 'style.css') ){
				$suffix = '';
				if( is_readable($dir_path . 'style-rtl.css') && is_rtl() ) {
					$suffix = '-rtl';
				}
				$this->styles[ $widget ]['uri'] = $uri . 'style'.$suffix.'.css';
				$this->styles[ $widget ]['dir'] = $dir_path . 'style'.$suffix.'.css';
				$this->styles[ $widget ]['widget_name'] = self::ASSET_PREFIX . $widget . self::ASSET_STYLE_SUFFIX;

			}
		}
	}


	/**
	 * Register elements widgets assets
	 *
	 * @since 2.0.0
	 */
	public function register_widgets_assets( $assets ){

		$this->get_styles_to_load_natural( $assets );

		$styles = [];

		foreach ( $this->styles as $widget => $style ) {

			$style_config = [
				'src'     => $style['uri'],
				'deps'    => [],
				'version'   => REY_CORE_VERSION,
			];

			if( ! empty($this->styles_to_load_natural_mode) && in_array( $style['widget_name'], $this->styles_to_load_natural_mode, true ) ){
				$style_config['in_header_if_natural'] = true;
			}

			$styles[ $style['widget_name'] ] = $style_config;
		}

		reyCoreAssets()->register_asset('styles', $styles);

		$scripts = [];

		foreach ( $this->scripts as $widget => $script ) {
			$scripts[ self::ASSET_PREFIX . $widget . self::ASSET_SCRIPT_SUFFIX ] = [
				'src'     => $script,
				'deps'    => ['elementor-frontend', 'reycore-elementor-frontend'],
				'version'   => REY_CORE_VERSION,
			];
		}

		reyCoreAssets()->register_asset('scripts', $scripts);
	}

	/**
	 * Force newly added elements to insert into header
	 * after page was saved in Elementor.
	 *
	 * @since 2.0.0
	 */
	public function force_newly_added_elements_into_header( $post_css, $element ){

		if ( $post_css instanceof Elementor\Core\DynamicTags\Dynamic_CSS ) {
			return;
		}

		$el_name = $element->get_unique_name();

		// check if widget is used in this particular post.
		if( strpos($el_name, self::PREFIX) === false ){
			return;
		}

		$el_name_short = str_replace(self::PREFIX, '', $el_name);

		if( isset($this->styles[ $el_name_short ]) && $stylesheet = $this->styles[ $el_name_short ] ){

			$post_id = $post_css->get_post_id();

			if( ! isset($this->added_elements[$post_id]) ){
				$this->added_elements[$post_id] = [];
			}

			if( ! in_array($el_name, $this->added_elements[$post_id]) ){
				reyCoreAssets()->add_styles($stylesheet['widget_name']);
				$this->added_elements[$post_id][] = $el_name;
			}
		}
	}

	public function add_section_custom_css_to_post_file( $post_css, $element ){

		if ( $post_css instanceof Elementor\Core\DynamicTags\Dynamic_CSS ) {
			return;
		}

		if( 'section' !== $element->get_type() ){
			return;
		}

		$rey_custom_css = $element->get_settings('rey_custom_css');

		if( ! ($css = trim( $rey_custom_css )) ) {
			return;
		}

		$css = str_replace( 'SECTION-ID', $post_css->get_element_unique_selector( $element ), $css );

		$post_css->get_stylesheet()->add_raw_css( $css );
	}

	public function load_ajax_assets( $post_id, $instance ){

		$document = $instance->documents->get( $post_id );

		if ( $document ) {
			$data = $document->get_elements_data();
		}

		if ( empty( $data ) ) {
			return;
		}

		$_a = [];

		$assets = $this;

		$instance->db->iterate_data( $data, function( $element ) use ($assets, &$_a) {

			if ( empty( $element['widgetType'] ) ){
				return;
			}

			if( ! (strpos($element['widgetType'], $assets::PREFIX) !== false) ){
				return;
			}

			$name = str_replace($assets::PREFIX, '', $element['widgetType']);

			if( isset($assets->styles[$name]) ){
				$_a['styles'][$name] = $assets->styles[$name]['uri'];
			}

			if( isset($assets->scripts[$name]) ){
				$_a['scripts'][$name] = $assets->scripts[$name];
			}

		});

		if( !empty($_a) ){
			printf( "<div data-assets='%s'></div>", wp_json_encode($_a) );
		}
	}

	public function get_scripts(){
		return $this->scripts;
	}


	/**
	 * Retrieve the reference to the instance of this class
	 * @return ReyCoreElementor_Widget_Assets
	 */
	public static function getInstance()
	{
		if ( is_null( self::$_instance ) || ! ( self::$_instance instanceof self ) ) {
			self::$_instance = new self;
		}
		return self::$_instance;
	}
}

ReyCoreElementor_Widget_Assets::getInstance();
