<?php

namespace ReyCore\Ajax;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Base
{
	const NONCE_KEY = 'reycore_ajax';

	/**
	 * HTTP status code for bad request error.
	 */
	const BAD_REQUEST = 400;

	/**
	 * HTTP status code for unauthorized access error.
	 */
	const UNAUTHORIZED = 401;

	/**
	 * Ajax actions.
	 *
	 * Holds all the register ajax action.
	 *
	 * @since 2.2.2
	 * @access private
	 *
	 * @var array
	 */
	private $ajax_actions = [];

	/**
	 * Ajax requests.
	 *
	 * Holds all the register ajax requests.
	 *
	 * @since 2.2.2
	 * @access private
	 *
	 * @var array
	 */
	private $requests = [];

	/**
	 * Ajax response data.
	 *
	 * Holds all the response data for all the ajax requests.
	 *
	 * @since 2.2.2
	 * @access private
	 *
	 * @var array
	 */
	private $response_data = [];

	/**
	 * Current ajax action ID.
	 *
	 * Holds all the ID for the current ajax action.
	 *
	 * @since 2.2.2
	 * @access private
	 *
	 * @var string|null
	 */
	private $current_action_id = null;

	private static $_instance = null;

	/**
	 * Ajax manager constructor.
	 *
	 * Initializing ajax manager.
	 *
	 * @since 2.2.2
	 * @access public
	 */
	private function __construct() {
		add_action( 'wp_ajax_reycore_ajax', [ $this, 'handle_ajax_request' ] );
		add_action( 'wp_ajax_nopriv_reycore_ajax', [ $this, 'handle_ajax_request' ] );
		add_filter( 'reycore/script_params', [ $this, 'add_script_params']);
	}

	/**
	 * Add script params.
	 *
	 * @since 2.2.2
	 * @access public
	 *
	 * @param array   $params
	 */
	public function add_script_params( $params ) {
		return array_merge($params, [
			'ajax_url' => admin_url( 'admin-ajax.php' ),
			'ajax_nonce' => wp_create_nonce( self::NONCE_KEY ),
		]);
	}

	/**
	 * Register ajax action.
	 *
	 * Add new actions for a specific ajax request and the callback function to
	 * be handle the response.
	 *
	 * Auth levels
	 * 1 = logged-in only
	 * 2 = logged-out only
	 * 3 = both logged-in and out
	 *
	 * @since 2.2.2
	 * @access public
	 *
	 * @param string   $tag      Ajax request name/tag.
	 * @param callable $callback The callback function.
	 */
	public function register_ajax_action( $tag, $callback, $auth_level = 3 ) {

		if ( ! did_action( 'reycore/ajax/register_actions' ) ) {
			_doing_it_wrong( __METHOD__, esc_html( sprintf( 'Use `%s` hook to register ajax action.', 'reycore/ajax/register_actions' ) ), '2.2.2' );
		}

		$this->ajax_actions[ $tag ] = [
			'callback' => $callback,
			'auth' => $auth_level,
		];

	}

	/**
	 * Handle ajax request.
	 *
	 * Verify ajax nonce, and run all the registered actions for this request.
	 *
	 * Fired by `wp_ajax_reycore_ajax` action.
	 *
	 * @since 2.2.2
	 * @access public
	 */
	public function handle_ajax_request() {

		if ( ! $this->verify_request_nonce() ) {
			$this->add_response_data( false, esc_html__( 'Token Expired or Invalid.', 'rey-core' ) )->send_error( self::UNAUTHORIZED );
		}

		/**
		 * Register ajax actions.
		 *
		 * Fires when an ajax request is received and verified.
		 *
		 * Used to register new ajax action handles.
		 *
		 * @since 2.2.2
		 *
		 * @param self $this An instance of ajax manager.
		 */
		do_action( 'reycore/ajax/register_actions', $this );

		if( ! (isset($_REQUEST['rey_action_id']) && ($action_id = reycore__clean($_REQUEST['rey_action_id']))) ){
			$this->add_response_data( false, esc_html__( 'Missing action.', 'rey-core' ) )->send_error( self::BAD_REQUEST );
		}

		if ( ! isset( $this->ajax_actions[ $action_id ] ) ) {
			$this->add_response_data( false, esc_html__( 'Action not found.', 'rey-core' ) )->send_error( self::UNAUTHORIZED );
		}

		$this->current_action_id = $action_id;

		if ( ! isset( $this->ajax_actions[ $action_id ]['auth'] ) ) {
			$this->add_response_data( false, esc_html__( 'Missing auth level.', 'rey-core' ) )->send_error( self::UNAUTHORIZED );
		}
		else {
			$logged_status = is_user_logged_in();

			if( $this->ajax_actions[ $action_id ]['auth'] === 1 && ! $logged_status ){
				$this->add_response_data( false, esc_html__( 'Incorrect authorization level.', 'rey-core' ) )->send_error( self::UNAUTHORIZED );
			}
			elseif( $this->ajax_actions[ $action_id ]['auth'] === 2 && $logged_status ){
				$this->add_response_data( false, esc_html__( 'Incorrect authorization level.', 'rey-core' ) )->send_error( self::UNAUTHORIZED );
			}
		}

		try {

			$action_data = isset($_REQUEST['rey_action_data']) ? reycore__clean($_REQUEST['rey_action_data']) : [];

			$results = call_user_func( $this->ajax_actions[ $action_id ]['callback'], $action_data, $this );

			if ( false === $results ) {
				$this->add_response_data( false );
			} else {
				$this->add_response_data( true, $results );
			}

		} catch ( \Exception $e ) {
			$this->add_response_data( false, $e->getMessage() )->send_error( $e->getCode() );
		}

		$this->current_action_id = null;

		$this->send_success();
	}

	/**
	 * Verify request nonce.
	 *
	 * Whether the request nonce verified or not.
	 *
	 * @since 2.2.2
	 * @access public
	 *
	 * @return bool True if request nonce verified, False otherwise.
	 */
	public function verify_request_nonce() {
		return ! empty( $_REQUEST['_nonce'] ) && wp_verify_nonce( $_REQUEST['_nonce'], self::NONCE_KEY );
	}

	/**
	 * Add response data.
	 *
	 * Add new response data to the array of all the ajax requests.
	 *
	 * @since 2.2.2
	 * @access protected
	 *
	 * @param bool  $success True if the requests returned successfully, False
	 *                       otherwise.
	 * @param mixed $data    Optional. Response data. Default is null.
	 *
	 * @param int   $code    Optional. Response code. Default is 200.
	 *
	 * @return Base An instance of ajax manager.
	 */
	private function add_response_data( $success, $data = null, $code = 200 ) {

		$this->response_data[ $this->current_action_id ] = [
			'success' => $success,
			'code' => $code,
			'data' => $data,
		];

		return $this;
	}

	/**
	 * Ajax failure response.
	 *
	 * Send a JSON response data back to the ajax request, indicating failure.
	 *
	 * @since 2.2.2
	 * @access protected
	 *
	 * @param null $code
	 */
	private function send_error( $code = null ) {
		wp_send_json_error( $this->response_data, $code );
	}

	/**
	 * Ajax success response.
	 *
	 * Send a JSON response data back to the ajax request, indicating success.
	 *
	 * @since 2.2.2
	 * @access protected
	 */
	private function send_success() {

		$response = [
			'success' => true,
			'data' => $this->response_data,
		];

		$json = wp_json_encode( $response );

		while ( ob_get_status() ) {
			ob_end_clean();
		}

		if ( function_exists( 'gzencode' ) ) {
			$response = gzencode( $json );

			header( 'Content-Type: application/json; charset=utf-8' );
			header( 'Content-Encoding: gzip' );
			header( 'Content-Length: ' . strlen( $response ) );

			echo $response; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		} else {
			echo $json; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		}

		wp_die( '', '', [ 'response' => null ] );
	}

	/**
	 * Retrieve the reference to the instance of this class
	 * @return Base
	 */
	public static function getInstance()
	{
		if ( is_null( self::$_instance ) || ! ( self::$_instance instanceof self ) ) {
			self::$_instance = new self;
		}
		return self::$_instance;
	}

}

Base::getInstance();
