<?php
namespace ReyCore\Cards;

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class Base
{
	private static $_instance = null;

	const ASSET_HANDLE = 'reycore-cards';

	public static $defaults = [];

	public $cards = [];

	const DEFAULT_CARD = 'basic';
	const CARD_KEY = 'card';

	private function __construct()
	{
		$this->includes();

		add_action('init', [$this, 'init']);
	}

	public function is_enabled(){
		return ! (defined('REYCORE_CARDS_DISABLE') && REYCORE_CARDS_DISABLE);
	}

	public function includes(){

		require_once __DIR__ . '/card-item.php';

		foreach(array_filter( glob( __DIR__ . '/*'), 'is_dir' ) as $dir){

			if( ! $this->is_enabled() && self::DEFAULT_CARD !== basename($dir, '.php') ){
				continue;
			}

			if( ( $file = trailingslashit($dir) . 'card.php' ) && is_readable($file) ){
				require $file;
			}
		}

	}

	public function init() {

		if( ! $this->is_enabled() ){
			return;
		}

		self::$defaults = apply_filters('reycore/cards/settings', [
			'title_tag' => 'h2',
			'label_tag' => 'h3',
			'desc_tag' => 'div',
		]);

		do_action('reycore/cards/init', $this);

		add_action( 'reycore/assets/register_scripts', [$this, 'register_assets']);
		add_filter( 'reycore/critical_css/css', [$this, 'add_critical_css'], 10);
		add_action( 'elementor/widgets/widgets_registered', [$this, 'load_default_card'], 9 );
	}

	public function register( $class ){

		if( ! ($card_id = $class->get_id()) ){
			return;
		}

		$this->cards[ $card_id ] = $class;
	}

	public function register_assets( $manager ){

		foreach ($this->cards as $id => $card) {

			foreach ([
				'get_css' => 'styles',
				'get_js'  => 'scripts',
			] as $func => $handle) {

				if( $manager && ($assets = $card->$func()) && is_array($assets) ){
					$manager->register_asset($handle, $assets);
				}
			}
		}

	}

	public function get_cards_list($cards = []){

		if( empty($this->cards) ){
			return $cards;
		}

		foreach ($this->cards as $id => $card) {
			$cards[ $id ] = $card->get_name();
		}

		return $cards;
	}

	public function get_card( $card ){

		if( ! isset( $this->cards[ $card ] ) ){
			return new \ReyCore\Cards\Card;
		}

		return $this->cards[ $card ];
	}

	public function load_default_card(){
		require_once __DIR__ . '/card-element.php';
	}

	public function get_card_supports( $type ){

		$supports = [];

		foreach ($this->cards as $card) {

			if( ! in_array( $type, (array) $card->get_supports(), true ) ){
				continue;
			}

			$supports[] = $card->get_id();
		}

		return $supports;
	}

	public function add_cards_controls( $element ){
		foreach ($this->cards as $card) {
			$card->get_card_controls( $element );
		}
	}

	public function render_card( $element ){

		if( ! isset($element->_settings[self::CARD_KEY]) ){
			return;
		}

		$card_id = $element->_settings[self::CARD_KEY];

		if( ! isset( $this->cards[ $card_id ] ) ){
			return;
		}

		$this->cards[ $card_id ]->render( $element );
	}

	public function add_critical_css( $css ){

		foreach ($this->cards as $id => $card) {
			if( $cc = (array) $card->get_critical_css() ){
				$css = array_merge($css, $cc);
			}
		}

		return $css;
	}

	/**
	 * Retrieve the reference to the instance of this class
	 * @return Base
	 */
	public static function getInstance()
	{
		if ( is_null( self::$_instance ) || ! ( self::$_instance instanceof self ) ) {
			self::$_instance = new self;
		}
		return self::$_instance;
	}

}

function card_instance(){
	return Base::getInstance();
}

card_instance();
