<?php
if (!defined('ABSPATH')) exit; // Exit if accessed directly

if( class_exists('WooCommerce') && !class_exists('ReyCore_Wc_PriceFeatures') ):

class ReyCore_Wc_PriceFeatures
{
	private $settings = [];

	private $product;

	const ASSET_HANDLE = 'reycore-price-features';

	public function __construct()
	{
		add_action( 'reycore/kirki_fields/after_field=single_product_price_text_inline', [ $this, 'add_customizer_options' ] );
		add_action( 'wp', [$this, 'init']);
	}

	public function init()
	{
		if( ! is_product() ){
			return;
		}

		$this->product = wc_get_product();

		if( ! $this->product ){
			return;
		}

		$this->settings = apply_filters('reycore/module/price_features', [
			'force_is_purchasable' => $this->product->is_purchasable()
		]);

		if( ! $this->is_enabled() ){
			return;
		}

		add_action( 'reycore/assets/register_scripts', [$this, 'register_assets']);
		add_action( 'wp_enqueue_scripts', [ $this, 'enqueue_scripts' ] );
		add_filter( 'body_class', [$this, 'add_hide_variation_price_class']);

	}

	function add_hide_variation_price_class($classes){

		if( $this->price_variation_change__enabled() ){
			$classes[] = '--hide-variation-price';
		}

		return $classes;
	}

	public static function load_scripts(){
		reyCoreAssets()->add_scripts(['wnumb', self::ASSET_HANDLE]);
		reyCoreAssets()->add_styles(self::ASSET_HANDLE);
	}

	public function enqueue_scripts(){
		if( is_product() ){
			self::load_scripts();
		}
	}

	public function register_assets(){

		reyCoreAssets()->register_asset('styles', [
			self::ASSET_HANDLE => [
				'src'     => REY_CORE_MODULE_URI . basename(__DIR__) . '/style.css',
				'deps'    => [],
				'version'   => REY_CORE_VERSION,
			]
		]);

		$params = [
			'variation_price_to_main'  => $this->price_variation_change__enabled(),
			'price_show_total'         => $this->price_add_total__enabled(),
			'price_instalments'        => $this->price_instalments__enabled(),
			'price_instalments_number' => get_theme_mod('single_product_price__instalments_number', 6),
			'price_instalments_text'   => __('Pay %p in %i instalments.', 'rey-core'),
		];

		if( $custom_text = get_theme_mod('single_product_price__instalments_text') ){
			$params['price_instalments_text'] = $custom_text;
		}

		reyCoreAssets()->register_asset('scripts', [
			self::ASSET_HANDLE => [
				'src'     => REY_CORE_MODULE_URI . basename(__DIR__) . '/script.js',
				'deps'    => ['rey-script', 'reycore-scripts', 'reycore-woocommerce'],
				'version'   => REY_CORE_VERSION,
				'localize' => [
					'name' => 'reycorePriceFeaturesParams',
					'params' => $params,
				],
			]
		]);

	}

	function add_customizer_options($field_args){

		ReyCoreKirki::add_field( 'rey_core_kirki', [
			'type'        => 'toggle',
			'settings'    => 'single_product_price__variations',
			'label'       => esc_html__( 'Change main price in Variable products', 'rey-core' ),
			'section'     => $field_args['section'],
			'default'     => false,
			'active_callback' => [
				[
					'setting'  => 'single_product_price',
					'operator' => '==',
					'value'    => true,
				],
			],
			'input_attrs' => [
				'data-control-class' => '--separator-top',
			]
		] );

		ReyCoreKirki::add_field( 'rey_core_kirki', [
			'type'        => 'toggle',
			'settings'    => 'single_product_price__show_total',
			'label'       => esc_html__( 'Add Total based on Quantity', 'rey-core' ),
			'section'     => $field_args['section'],
			'default'     => false,
			'active_callback' => [
				[
					'setting'  => 'single_product_price',
					'operator' => '==',
					'value'    => true,
				],
			],

		] );

		ReyCoreKirki::add_field( 'rey_core_kirki', [
			'type'        => 'toggle',
			'settings'    => 'single_product_price__instalments',
			'label'       => esc_html__( 'Add payment in instalments text.', 'rey-core' ),
			'section'     => $field_args['section'],
			'default'     => false,
			'active_callback' => [
				[
					'setting'  => 'single_product_price',
					'operator' => '==',
					'value'    => true,
				],
			],
		] );

		ReyCoreKirki::add_field( 'rey_core_kirki', [
			'type'        => 'rey-number',
			'settings'    => 'single_product_price__instalments_number',
			'label'       => esc_html__( 'Number of instalments', 'rey-core' ),
			'section'     => $field_args['section'],
			'default'     => 6,
			'choices'     => [
				'min'  => 1,
				'max'  => 100,
				'step' => 1,
			],
			'active_callback' => [
				[
					'setting'  => 'single_product_price',
					'operator' => '==',
					'value'    => true,
				],
				[
					'setting'  => 'single_product_price__instalments',
					'operator' => '==',
					'value'    => true,
				],
			],
		] );

		ReyCoreKirki::add_field( 'rey_core_kirki', [
			'type'        => 'text',
			'settings'    => 'single_product_price__instalments_text',
			'label'       => esc_html__( 'instalments Text', 'rey-core' ),
			'section'     => $field_args['section'],
			'default'     => __('Pay %p in %i instalments.', 'rey-core'),
			'input_attrs'     => [
				'placeholder' => __('eg: Pay %p in %i instalments.', 'rey-core'),
				'data-control-class' => '--text-xl',
			],
			'sanitize_callback' => ['Kirki_Sanitize_Values', 'unfiltered'],
			'active_callback' => [
				[
					'setting'  => 'single_product_price',
					'operator' => '==',
					'value'    => true,
				],
				[
					'setting'  => 'single_product_price__instalments',
					'operator' => '==',
					'value'    => true,
				],
			],
		] );
	}

	public function price_variation_change__enabled(){
		return get_theme_mod('single_product_price__variations', false);
	}

	public function price_add_total__enabled(){
		return get_theme_mod('single_product_price__show_total', false);
	}

	public function price_instalments__enabled(){
		return get_theme_mod('single_product_price__instalments', false);
	}

	public function is_enabled() {
		return ($this->price_variation_change__enabled() || $this->price_add_total__enabled() || $this->price_instalments__enabled()) && $this->settings['force_is_purchasable'];
	}

}

new ReyCore_Wc_PriceFeatures;

endif;
