<?php

namespace ReyCore\ProductsPerPage;

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class Base
{
	private static $_instance = null;

	const ASSET_HANDLE = 'reycore-ppp';

	const COOKIE_NAME = 'reycore-ppp';

	private function __construct()
	{
		if( ! class_exists('WooCommerce') ){
			return;
		}

		add_action('init', [$this, 'init']);
		add_action( 'wp_ajax_reycore_set_ppp', [$this, 'set_ppp'] );
		add_action( 'wp_ajax_nopriv_reycore_set_ppp', [$this, 'set_ppp'] );

		$this->includes();
	}

	function is_enabled(){
		return get_theme_mod('loop_switcher_ppp', false);
	}

	public function init(){

		if( ! $this->is_enabled() ) {
			return;
		}

		add_action( 'reycore/assets/register_scripts', [$this, 'register_assets']);
		add_filter( 'reycore/loop_components', [$this, 'enable_component']);
		add_filter( 'reycore/loop/component_hooks', [$this, 'add_component']);
		add_action( 'woocommerce_product_query', [ $this, 'woocommerce_product_query'], 100);
	}

	function includes(){
		require_once __DIR__ . '/customizer.php';
	}

	function enable_component( $components ){
		$components['ppp_selector'] = true;
		return $components;
	}

	function add_component( $components ){

		$components['ppp_selector'] = [
			'type'          => 'action',
			'tag'           => 'reycore/woocommerce/loop/before_grid',
			'callback'      => [ $this, 'loop_component' ],
			'priority'      => 29,
		];

		return $components;
	}

	function get_layout(){
		return get_theme_mod('loop_switcher_ppp_layout', 'inline');
	}

	function get_settings(){

		$posts_per_page = reycore_wc_get_columns('desktop') * wc_get_default_product_rows_per_page();

		$settings = apply_filters('reycore/woocommerce/ppp_selector/settings', [
			'label' => esc_html_x('SHOW', 'Label for "show selector" in catalog', 'rey-core'),
			'options' => [
				$posts_per_page,
				$posts_per_page * 2,
				$posts_per_page * 4
			],
			'selected' => absint($posts_per_page),
		], $this);

		if ( ($custom_ppp = $this->get_ppp()) && isset($settings['options']) && in_array($custom_ppp, $settings['options'], true) ) {
			$settings['selected'] = $custom_ppp;
		}

		return $settings;
	}

	public function woocommerce_product_query( $q )
	{
		if ( ! ($custom_ppp = $this->get_ppp() ) ) {
			return;
		}

		if ( ! ( ($settings = $this->get_settings()) && isset($settings['options']) && in_array($custom_ppp, $settings['options'], true) ) ) {
			return;
		}

		$q->set( 'posts_per_page', $custom_ppp );
	}

	public function loop_component()
	{
		$settings = $this->get_settings();

		reycore__get_template_part('template-parts/woocommerce/ppp-selector-' . $this->get_layout(), false, false, [
			'label' => $settings['label'],
			'options' => $settings['options'],
			'selected' => $settings['selected'],
		]);

		reyCoreAssets()->add_styles(self::ASSET_HANDLE);
		reyCoreAssets()->add_scripts(self::ASSET_HANDLE);
	}

	public function get_ppp(){

		$sel = 0;

		if ( isset($_COOKIE[self::COOKIE_NAME]) && ! empty( $_COOKIE[self::COOKIE_NAME] ) ) {
			$sel = reycore__clean( wp_unslash( $_COOKIE[self::COOKIE_NAME] ) );
		}

		return absint($sel);
	}

	function set_ppp(){

		if( ! $this->is_enabled() ) {
			return;
		}

		if ( ! check_ajax_referer( 'rey_nonce', 'security', false ) ) {
			wp_send_json_error( esc_html__('Invalid security nonce!', 'rey-core') );
		}

		if( ! ( isset($_REQUEST['ppp']) && $count = absint($_REQUEST['ppp']) ) ){
			wp_send_json_error( esc_html__('Product per page count not found.', 'rey-core') );
		}

		$settings = $this->get_settings();

		// force only predefined options to be allowed
		if( ! in_array($count, $settings['options'], true) ){
			wp_send_json_error( esc_html__('Invalid count.', 'rey-core') );
		}

		wc_setcookie(self::COOKIE_NAME, $count, time() + DAY_IN_SECONDS * 2);

		wp_send_json_success();
	}

	public function register_assets(){

		reyCoreAssets()->register_asset('styles', [
			self::ASSET_HANDLE => [
				'src'     => REY_CORE_MODULE_URI . basename(__DIR__) . '/frontend-style.css',
				'deps'    => ['woocommerce-general', 'rey-wc-product'],
				'version'   => REY_CORE_VERSION,
			],
		]);

		reyCoreAssets()->register_asset('scripts', [
			self::ASSET_HANDLE => [
				'src'     => REY_CORE_MODULE_URI . basename(__DIR__) . '/frontend-script.js',
				'deps'    => ['rey-script', 'reycore-scripts', 'reycore-woocommerce'],
				'version'   => REY_CORE_VERSION,
			],
		]);

	}

	public static function getInstance()
	{
		if ( is_null( self::$_instance ) || ! ( self::$_instance instanceof self ) ) {
			self::$_instance = new self;
		}
		return self::$_instance;
	}

}

function instance(){
	return Base::getInstance();
}

instance();
