<?php

namespace ReyCore\RequestQuote;

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class Base
{
	private static $_instance = null;

	public static $settings = [];

	private $defaults = [];

	private $type = '';

	const ASSET_HANDLE = 'reycore-request-quote';

	private function __construct()
	{

		if( ! class_exists('WooCommerce') ){
			return;
		}

		add_action('init', [$this, 'init']);

		$this->includes();

	}

	function get_type(){
		return get_theme_mod('request_quote__type', '');
	}

	function is_enabled(){
		return $this->get_type() !== '';
	}

	public function init(){

		if( ! $this->is_enabled() ) {
			return;
		}

		$this->type = $this->get_type();

		$this->set_defaults();

		add_action( 'reycore/assets/register_scripts', [$this, 'register_assets']);
		add_filter( 'reycore/modal_template/show', '__return_true' );
		add_action( 'rey/after_site_wrapper', [$this, 'add_form_modal'], 50);
		add_action( 'woocommerce_single_product_summary', [$this, 'get_button_html'], 30);
		add_shortcode( 'rey_request_quote', [$this, 'get_button_html']);

	}

	function includes(){
		require_once __DIR__ . '/customizer.php';
		require_once __DIR__ . '/compat-cf7.php';
		require_once __DIR__ . '/compat-wpforms.php';
	}

	public function register_assets(){

		reyCoreAssets()->register_asset('styles', [
			self::ASSET_HANDLE => [
				'src'     => REY_CORE_MODULE_URI . basename(__DIR__) . '/frontend-style.css',
				'deps'    => ['woocommerce-general', 'rey-wc-product'],
				'version'   => REY_CORE_VERSION,
			],
		]);

		reyCoreAssets()->register_asset('scripts', [
			self::ASSET_HANDLE => [
				'src'     => REY_CORE_MODULE_URI . basename(__DIR__) . '/frontend-script.js',
				'deps'    => ['rey-script', 'reycore-scripts', 'reycore-woocommerce'],
				'version'   => REY_CORE_VERSION,
				'localize' => [
					'name' => 'reycoreRequestQuoteParams',
					'params' => [
						'variation_aware' => $this->defaults['variation_aware'],
						'disabled_text' => $this->defaults['disabled_text']
					],
				],
			],
		]);

	}

	/**
	 * Set defaults
	 *
	 * @since 1.2.0
	 **/
	public function set_defaults()
	{
		$this->defaults = apply_filters('reycore/woocommerce/request_quote_defaults', [
			'title' => get_theme_mod('request_quote__btn_text', esc_html__( 'Request a Quote', 'rey-core' ) ),
			'product_title' => esc_html__( 'PRODUCT: ', 'rey-core' ),
			'close_position' => 'inside',
			'show_in_quickview' => false,
			'variation_aware' => get_theme_mod('request_quote__var_aware', false ),
			'disabled_text' => esc_html__('Please select some product options before requesting quote.', 'rey-core')
		]);
	}

	public function maybe_show(){
		return apply_filters('reycore/woocommerce/request_quote/display', $this->__check_display(), $this);
	}

	private function __check_display(){

		if( $this->type === 'products' ){

			if( ! ($products = get_theme_mod('request_quote__products', '')) ){
				return false;
			}

			$get_products_ids = array_map( 'absint', array_map( 'trim', explode( ',', $products ) ) );

			if( ! in_array(get_the_ID(), $get_products_ids) ){
				return false;
			}
		}

		elseif( $this->type === 'categories' ){

			if( ! ($categories = get_theme_mod('request_quote__categories', [])) ){
				return false;
			}

			if( $product = reycore_wc__get_product() ){
				if( ! has_term($categories, 'product_cat', $product->get_id()) ){
					return false;
				}
			}
		}

		if( get_query_var('rey__is_quickview', false) === true && $this->defaults['show_in_quickview'] === false ) {
			return false;
		}

		return true;
	}

	public function add_form_modal(){

		if( ! reycore_wc__is_product() ){
			return;
		}

		if( ! $this->maybe_show() ){
			return;
		}

		$form = apply_filters('reycore/woocommerce/request_quote/output', '', [
			'class' => 'rey-form--basic'
		] );

		if( empty($form) ){
			return;
		}

		reycore__get_template_part('template-parts/woocommerce/request-quote-modal', false, false, [
			'form' => $form,
			'defaults' => $this->defaults
		]);

	}

	/**
	* Add the button
	*
	* @since 1.2.0
	*/
	public function get_button_html( $args = [] ){

		if( ! $this->maybe_show() ){
			return;
		}

		if( ! empty($args) ){
			$this->defaults = array_merge($this->defaults, $args);
		}

		reycore__get_template_part('template-parts/woocommerce/request-quote-button', false, false, [
			'button_text' => $this->defaults['title']
		]);

		reyCoreAssets()->add_styles(self::ASSET_HANDLE);
		reyCoreAssets()->add_scripts(self::ASSET_HANDLE);

		// load modal scripts
		add_filter('reycore/modals/always_load', '__return_true');
	}

	/**
	 * Retrieve the reference to the instance of this class
	 * @return Base
	 */
	public static function getInstance()
	{
		if ( is_null( self::$_instance ) || ! ( self::$_instance instanceof self ) ) {
			self::$_instance = new self;
		}
		return self::$_instance;
	}

}

function instance(){
	return Base::getInstance();
}

instance();
